/* $Id: callbacks_bak.c 455 2003-12-30 18:58:10Z michael $ */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gdk/gdkx.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "cspec.h"			/* stuctures used by more than one file */
#include "callbacks.h"		/*								*/
#include "interface.h"		/* These are generated by glade */
#include "support.h"		/*								*/

#include "centroid.h"		/* compute the center of a feature */
#include "genbc.h"			/* generates coefficients to estimate f(x) */
#include "tickmks.h"		/* calculates positions for tick marks on plot */
#include "img2ps.h"			/* converts JPEG to EPS */
#include "fwhm.h"			/* calculates the FWHM for the given reference line and mark */
#include "logfile.h"		/* handles logging to a text file */

#define BORDER 100			/* size of border around plot */
#define NORMAL_CURSOR 0		/* small crosshair cursor */
#define CROSSHAIR_CURSOR 1	/* large line cursor */
#define PRINT_TMP_FILE ".tmp.eps"
#define PRETTY_BIG_NUM 4096

struct plotarea		parea;
GList				*spectra = NULL;		/* This should always point to the first element of the list */
GList				*active_spectrum;
GList				*colorlist = NULL,
					*ncolorlist = NULL,
					*icolorlist = NULL;

struct line			pix2wvl,
					pix2flux,
					flux2pix,
					wvl2pix,
					refline;

static GtkWidget	*open_filesel = NULL,
					*dialog2 = NULL,
					*dialog3 = NULL,
					*printdialog = NULL,
					*close_dialog = NULL,
					*scale_dialog = NULL,
					*help_window = NULL,
					*message_dialog = NULL,
					*main_window,
					*label13,
					*label15,
					*spinbutton1,
					*spinbutton2,
  				 	*statusbar,
					*xminspin,
					*xmaxspin,
					*yminentry,
					*ymaxentry;

gchar 				label13_text[64],
					label15_text[64],
					stattxt[PATH_MAX];

const char			*filename;

static const gchar	*MainWindowKey = "MainWindowKey";

gdouble				a=0.,
					b=100000,
					mark=0.,
					increment,
					i=0.0;

gint				xzoom1=0, xzoom2=0,
					yzoom1=0, yzoom2=0,
					ymax,
					anum_samples=0,
					cursor_type = NORMAL_CURSOR,
					cursor_x,
					cursor_y;

gboolean			one_pressed = FALSE,
					show_guides = TRUE,
					printed_header = FALSE;

GdkColor			*zoom_color,
					*bg_color,
					*ab_color,
					*ref_color,
					*mark_color,
					*cursor_color,
					*axes_color,
					*black_color,
					*blue_color,
					*red_color,
					*yellow_color,
					*green_color,
					*grey_color,
					*purple_color,
					*orange_color,
					*white_color;

/* Static functions. */
void open_file (GtkWidget *main_window);
void real_open_file (GtkWidget *main_window );

/* function */
void draw_plot( GdkDrawable *widget );
void erase_plot( GtkWidget *widget );
gdouble x2wvl( gint x );
gdouble y2flux( gint y );
void set_defaults();
void clean_up_and_quit();
void update_labels();
void swap_ints( gint* n1, gint* n2 );
void increment_cont_slope( GtkWidget *widget, gint inc );
void increment_cont_inter( GtkWidget *widget, gint inc );
void integrate();
void do_save_as();
void do_print();
int normal_colors();
int inverted_colors();
void save_image( char* sname, char* stype, int width, int height );
void greyout( gboolean state );
void move_pointer( int xinc, int yinc );
struct spectdata*  init_sdata( struct spectdata *sdata );
void draw_spectrum( struct spectdata *sdata, GdkDrawable *widget, GdkGC *gc, int xrange, int yrange, struct line xform );
void free_node( GList *node );
void clear_button( gint n, GtkButton *button );
void update_status( char *message );
void report_fwhm( struct full_width_half_max fwhm );
void do_fwhm();
void display_message( char* message );
void mark_point( gdouble wvl, gdouble flux );
void draw_fwhm_line( gdouble left, gdouble right, struct spectdata *sdata );

/*
 * Window Manager Close Box Callback Test
 */

gboolean
on_main_window_delete_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{

	clean_up_and_quit();
	return FALSE;
}

/*
 * Menu Bar Callbacks
 */

void
on_open_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	open_file (main_window);
}

void
on_zoomout_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	set_defaults();
	erase_plot( main_window );
}


void
on_quit_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	clean_up_and_quit();
}

void
on_zoombutton_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	set_defaults();
	erase_plot( main_window );
}

gboolean
on_open_filesel_delete_event           (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	gtk_widget_hide (gtk_widget_get_toplevel (widget));
	return TRUE;
}


void
on_open_filesel_ok_button_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *filesel, *main_window;
                                                                                     
	filesel = gtk_widget_get_toplevel (GTK_WIDGET (button));
	main_window = gtk_object_get_data (GTK_OBJECT (filesel), MainWindowKey);
	gtk_widget_hide (filesel);
	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
	real_open_file ( main_window );
}


void
on_open_filesel_cancel_button_clicked  (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}

/* This shows the file selection dialog to open a file. */
void open_file (GtkWidget *main_window)
{
  /* We use the same file selection widget each time, so first
     of all we create it if it hasn't already been created. */
  if (open_filesel == NULL)
    {
      open_filesel = create_open_file_selection ();
      /* Make sure the dialog doesn't disappear behind the main window. */
      gtk_window_set_transient_for (GTK_WINDOW (open_filesel),
                                    GTK_WINDOW (main_window));
    }
                                                                                     
  /* We save a pointer to the main window inside the file selection's
     data list, so we can get it easily in the callbacks. */
  gtk_object_set_data (GTK_OBJECT (open_filesel), MainWindowKey, main_window);
                                                                                     
  /* We make sure the dialog is visible. */
  gtk_window_present (GTK_WINDOW (open_filesel));
}

/* This loads the given file, which is in on-disk encoding (which may not
   be UTF-8). */
void real_open_file (GtkWidget *main_window ) {
	gint 				rtn,
						n;
	gchar 				*tmp_ptr;
	FILE				*infile;
	double				x[PRETTY_BIG_NUM], y[PRETTY_BIG_NUM];
	struct spectdata	*sdata = NULL;
                                                                                     
	statusbar = lookup_widget (main_window, "statusbar1");
	update_status( "Processing file..." );

	/* This will need to come back once we can replace existing open files */
	/* uni_free( sdata->ufile ); */

	if( g_list_length( spectra ) >= 6 ) {
		display_message( "Error: Can't open more than 6 files.");
		return;
	}
	sdata = init_sdata( sdata );

	/* first see if it is unifspec format */
	rtn = uni_read_file( filename, sdata->ufile );
	if (rtn != 0 ) {
		/* if that failed check if it is a text format */
		infile = fopen( filename, "r" );
		if( infile == NULL ) {
			display_message( "Error: Could not open file." );
			return;
		}
		n=0;
		// FIXME Will ignore any lines after the first PRETTY_BIG_NUM
		while( !feof( infile) && n < PRETTY_BIG_NUM ) {
			fscanf( infile, "%lf %lf\n", &x[n], &y[n] );
			++n;
    	}
    	fclose( infile );

		rtn = uni_create ( &x[0], &y[0], n, n, sdata->ufile );
	}

	/* check to see if we are happy */
	if( rtn == 0  ) {
		if( sdata->coeff_b != NULL )
			free( sdata->coeff_b );
		sdata->coeff_b = calloc( sdata->ufile->num_elems, sizeof( gdouble) );
		if( sdata->coeff_c != NULL )
			free( sdata->coeff_c );
		sdata->coeff_c = calloc( sdata->ufile->num_elems, sizeof( gdouble) );
		genbc( sdata->ufile->num_elems, sdata->ufile->data, sdata->coeff_b, sdata->coeff_c);

		sprintf( stattxt, "File \"%s\": %li data pts", filename, sdata->ufile->num_elems );
		writelog( stattxt );
		sprintf( stattxt, "x = %lg to %lg, dx = %lg", sdata->ufile->x1, sdata->ufile->xn, sdata->ufile->dx );
		writelog( stattxt );
		sprintf( stattxt, "ymin = %lg,  ymax = %lg.", sdata->ufile->ymin, sdata->ufile->ymax );
		writelog( stattxt );

		tmp_ptr = strrchr( filename, '/' ); tmp_ptr++;
		sprintf( stattxt, "cspec : %s", tmp_ptr );
		gtk_window_set_title( GTK_WINDOW(main_window), stattxt );

		sdata->filename = malloc( strlen( tmp_ptr )+1 );
		strncpy( sdata->filename, tmp_ptr, strlen( tmp_ptr )+1 );
		spectra = g_list_append( spectra, sdata );
		active_spectrum = g_list_find( spectra, sdata );

		set_defaults();
		update_labels();

		sprintf( stattxt, "File %s opened.", filename );
		update_status( stattxt );

		greyout( TRUE );
	
		GtkWidget *draw = lookup_widget(GTK_WIDGET(main_window), "drawingarea1");
		erase_plot( draw );
	}
	else
		display_message( "Error: Could not open file.");
}

void greyout( gboolean state ) {
		gtk_widget_set_sensitive( lookup_widget(GTK_WIDGET(main_window), "continuum_line1"), state );
		gtk_widget_set_sensitive( lookup_widget (GTK_WIDGET (main_window), "togglebutton1"), state );
		gtk_widget_set_sensitive( lookup_widget(GTK_WIDGET(main_window), "zoomout"), state );
		gtk_widget_set_sensitive( lookup_widget (GTK_WIDGET (main_window), "zoombutton"), state );
		gtk_widget_set_sensitive( lookup_widget(GTK_WIDGET(main_window), "invert1"), state );
		gtk_widget_set_sensitive( lookup_widget (GTK_WIDGET (main_window), "togglebutton3"), state );
		gtk_widget_set_sensitive( lookup_widget(GTK_WIDGET(main_window), "cursor_mode1"), state );
		gtk_widget_set_sensitive( lookup_widget (GTK_WIDGET (main_window), "cursortoggle"), state );
		gtk_widget_set_sensitive( lookup_widget(GTK_WIDGET(main_window), "integrate1"), state );
		gtk_widget_set_sensitive( lookup_widget (GTK_WIDGET (main_window), "integrate_button"), state );
		gtk_widget_set_sensitive( lookup_widget(GTK_WIDGET(main_window), "fwhm1"), state );
		gtk_widget_set_sensitive( lookup_widget (GTK_WIDGET (main_window), "fwhm_button"), state );
		gtk_widget_set_sensitive( lookup_widget(GTK_WIDGET(main_window), "print1"), state );
}

gboolean
on_drawingarea1_expose_event           (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
	draw_plot( GDK_DRAWABLE( widget->window ) );
	return FALSE;
}

void draw_plot( GdkDrawable *widget ) { 
	gint					y, y2, x, x2, width, height,
							fwidth, fheight,
							xmin, xmax, ymin, xrange, yrange, n, i;
	gchar					labeltxt[64];
	GdkGC 					*gc;
	PangoFontDescription	*font;
	PangoLayout  			*layout;
  	double					xticks[64];  
	GList					*current_spectrum;
	struct spectdata		*sdata;


	if( spectra != NULL ) {
		gdk_drawable_get_size( GDK_DRAWABLE( widget ), &width, &height );

		xmin = BORDER;
		xmax = width-BORDER;
		ymin = BORDER;
		ymax = height - BORDER;
		xrange = xmax - xmin;
		yrange = ymax - ymin;

		/* transform from wavelengths to pixels */
		wvl2pix.m = (gdouble) (xrange/(parea.wvlmax-parea.wvlmin));
		wvl2pix.b  = BORDER - (wvl2pix.m*parea.wvlmin);

		/* transform from pixels to wavelengths */
		pix2wvl.m = (parea.wvlmax-parea.wvlmin)/xrange;
		pix2wvl.b = parea.wvlmin - (pix2wvl.m*BORDER);

		/* transform from flux to pixels */
		flux2pix.m = -yrange/(parea.fluxmax-parea.fluxmin);
		flux2pix.b = BORDER - (flux2pix.m*parea.fluxmax);

		/* transform from pixels to flux */
		pix2flux.m = -(parea.fluxmax-parea.fluxmin)/yrange;
		pix2flux.b = parea.fluxmax - (pix2flux.m*BORDER);

		/* set up new graphics context */
		gc = gdk_gc_new( GDK_DRAWABLE( widget ) );
		gdk_gc_set_background( gc, bg_color);

		/* erase drawable */
		gdk_gc_set_foreground( gc, bg_color);
		gdk_draw_rectangle( GDK_DRAWABLE( widget ), gc, TRUE, 0, 0, width, height );

		/* draw bounding box for spectrum */
		gdk_gc_set_foreground( gc, axes_color);
		gdk_draw_rectangle( GDK_DRAWABLE( widget ), gc, FALSE, BORDER, BORDER, xrange, yrange );

		/* draw ticks */
		gdk_gc_set_foreground( gc, axes_color);
		GtkWidget *draw = lookup_widget(GTK_WIDGET( main_window ), "drawingarea1");
		layout = gtk_widget_create_pango_layout ( draw, NULL );
   		font =  pango_font_description_from_string ("Helvetica,Medium 12");
   		pango_layout_set_font_description (layout, font);

		/* y ticks */
		n = tickmks( parea.fluxmin, parea.fluxmax, &xticks[0]); 
		for ( i=0; i < n; i++) {
			y = flux2pix.m*xticks[i] + flux2pix.b;
  			gdk_draw_line( widget, gc, BORDER-10, y, BORDER, y );
			sprintf( labeltxt, "%4.2e", xticks[i] );
   			pango_layout_set_text (layout, labeltxt, -1);
 			pango_layout_get_pixel_size (layout, &fwidth, &fheight);
   			gdk_draw_layout (GDK_DRAWABLE(widget), gc, BORDER-11-fwidth, y-(fheight/2), layout);
		}

		/* x ticks */
		n = tickmks( parea.wvlmin, parea.wvlmax, &xticks[0]); 
		for ( i=0; i < n; i++ ) {
			x = wvl2pix.m*xticks[i] + wvl2pix.b;
  			gdk_draw_line( widget, gc, x, BORDER+yrange, x, BORDER+yrange+10 );
			sprintf( labeltxt, "%7.2f", xticks[i] );
   			pango_layout_set_text( layout, labeltxt, -1 );
 			pango_layout_get_pixel_size( layout, &fwidth, &fheight );
   			gdk_draw_layout ( GDK_DRAWABLE(widget), gc, x-(fwidth/2), BORDER+yrange+11, layout );
		}
		/* end draw ticks */

		/* draw the spectra themselves */
		gint spect_num=0;
		pango_font_description_free (font);
   		font =  pango_font_description_from_string ("Helvetica,Medium 10");
   		pango_layout_set_font_description (layout, font);
		for( current_spectrum =  spectra; current_spectrum != NULL; current_spectrum = g_list_next(current_spectrum) ) {
			sdata = current_spectrum->data;
			gdk_gc_set_foreground( gc, (GdkColor*)g_list_nth_data( colorlist, spect_num ) );
			draw_spectrum( sdata, widget, gc, xrange, yrange, flux2pix );
			if( sdata == active_spectrum->data )
				sprintf( labeltxt, "* %s", sdata->filename );
			else
				sprintf( labeltxt, "%s", sdata->filename );
   			pango_layout_set_text( layout, labeltxt, -1 );
 			pango_layout_get_pixel_size( layout, &fwidth, &fheight );
   			gdk_draw_layout ( GDK_DRAWABLE(widget), gc, width-fwidth-BORDER, spect_num*fheight, layout );
			++spect_num;
		}
		gdk_gc_set_foreground( gc, black_color);
		/* end draw spectra themselves */
		pango_font_description_free (font);
   		font =  pango_font_description_from_string ("Helvetica,Medium 12");
   		pango_layout_set_font_description (layout, font);

		/* draw zoom box */

		if( one_pressed ) {
			if( xzoom2 < xzoom1 ) {
				x = xzoom2;
				x2 = xzoom1;
			}
			else {
				x = xzoom1;
				x2 = xzoom2;
			}
			if( yzoom2 < yzoom1 ) {
				y = yzoom2;
				y2 = yzoom1;
			}
			else {
				y = yzoom1;
				y2 = yzoom2;
			}
			gdk_gc_set_foreground( gc, zoom_color);
			gdk_gc_set_line_attributes( gc, 2, GDK_LINE_SOLID, GDK_CAP_NOT_LAST, GDK_JOIN_MITER);
			gdk_draw_rectangle (widget, gc, FALSE, x, y, x2-x, y2-y );
			gdk_gc_set_line_attributes( gc, 1, GDK_LINE_SOLID, GDK_CAP_NOT_LAST, GDK_JOIN_MITER);
		}
		/* end draw zoom box */
	
		if( show_guides ) {
			gdk_gc_set_foreground( gc, ref_color);
			/* draw continuum line */
			gdk_gc_set_line_attributes( gc, 1, GDK_LINE_ON_OFF_DASH, GDK_CAP_NOT_LAST, GDK_JOIN_MITER);
			x = BORDER;
			x2 = xrange+BORDER;
			y   = (((refline.m*(x2wvl(x))) + refline.b) * flux2pix.m) + flux2pix.b;
			while( (y < BORDER || y > (yrange+BORDER) ) && x<x2 ) {
				++x;
				if( x == x2 ) {
					y=-1;
					break;
				}
				y   = (((refline.m*(x2wvl(x))) + refline.b) * flux2pix.m) + flux2pix.b;
			}
			y2  = (((refline.m*(x2wvl(x2))) + refline.b) * flux2pix.m) + flux2pix.b;
			while( ( y2 < BORDER || y2 > (yrange+BORDER) ) && x2>BORDER ) {
				--x2;
				if( x2 == BORDER ) {
					y2=-1;
					break;
				}
				y2  = (((refline.m*(x2wvl(x2))) + refline.b) * flux2pix.m) + flux2pix.b;
			}
			if( (y != -1) && (y2 != -1) )
  				gdk_draw_line( widget, gc, x, y, x2, y2 );
			/* end draw continuum line */
	
			/* draw a & b lines */
			gdk_gc_set_foreground( gc, ab_color);
			if( a > 0  ) {
				x = (gint)round((wvl2pix.m*a)+wvl2pix.b);
				if( x>BORDER && x<BORDER+xrange ) {
					y   = BORDER;
					y2  = ymax;
  					gdk_draw_line( widget, gc, x, y, x, y2 );
					sprintf( labeltxt, "%7.2f", a );
   					pango_layout_set_text (layout, labeltxt, -1);
 					pango_layout_get_pixel_size (layout, &fwidth, &fheight);
   					gdk_draw_layout (GDK_DRAWABLE(widget), gc, x-(fwidth/2), BORDER+yrange+31, layout);
				}
			}
	
			if( b < 100000  ) {
				x = (gint)round((wvl2pix.m*b)+wvl2pix.b);
				if( x>BORDER && x<BORDER+xrange ) {
					y   = BORDER;
					y2  = ymax;
  					gdk_draw_line( widget, gc, x, y, x, y2 );
					sprintf( labeltxt, "%7.2f", b );
   					pango_layout_set_text (layout, labeltxt, -1);
 					pango_layout_get_pixel_size (layout, &fwidth, &fheight);
   					gdk_draw_layout (GDK_DRAWABLE(widget), gc, x-(fwidth/2), BORDER+yrange+31, layout);
				}
			}
			/* end a & b lines */

			/* draw mark */
			if( mark > 0 ) {
				gdk_gc_set_foreground( gc, mark_color);
				x = (gint)round((wvl2pix.m*mark)+wvl2pix.b);
				if( x>BORDER && x<BORDER+xrange ) {
					y   = BORDER;
					y2  = ymax;
  					gdk_draw_line( widget, gc, x, y, x, y2 );
					sprintf( labeltxt, "%7.2f", mark );
   					pango_layout_set_text (layout, labeltxt, -1);
 					pango_layout_get_pixel_size (layout, &fwidth, &fheight);
   					gdk_draw_layout (GDK_DRAWABLE(widget), gc, x-(fwidth/2), BORDER+yrange+31, layout);
				}
			}
			gdk_gc_set_line_attributes( gc, 1, GDK_LINE_SOLID, GDK_CAP_NOT_LAST, GDK_JOIN_MITER);
			/* end draw mark */
		}

		/* draw cursor */

		if( cursor_type == CROSSHAIR_CURSOR ) {
			gdk_gc_set_foreground( gc, cursor_color);
			if ( cursor_x < BORDER )
				cursor_x = BORDER;
			if ( cursor_x > BORDER+xrange )
				cursor_x = BORDER+xrange;
			if ( cursor_y < BORDER )
				cursor_y = BORDER;
			if ( cursor_y > BORDER+yrange )
				cursor_y = BORDER+yrange;

  			gdk_draw_line( widget, gc, BORDER, cursor_y, BORDER+xrange, cursor_y );
  			gdk_draw_line( widget, gc, cursor_x, BORDER, cursor_x, BORDER+yrange );
		}

		gdk_gc_set_foreground( gc, black_color);

		update_labels();

		g_object_unref (layout);
		g_object_unref (gc);
		pango_font_description_free (font);
	}
	return;
}

void erase_plot( GtkWidget *widget ) {
	/* this invalidates the drawing area, scheduling it for a redraw */
	gtk_widget_queue_draw_area( widget, 0, 0, (gint)widget->allocation.width, (gint)widget->allocation.height);
}

gboolean
on_drawingarea1_motion_notify_event    (GtkWidget       *widget,
                                        GdkEventMotion  *event,
                                        gpointer         user_data)
{
	gint			x,y;
	gdouble			nx, ny;
	GdkModifierType	state;

	if (event->is_hint) {
		gdk_window_get_pointer(event->window, &x, &y, &state);
	} else {
		x = (gint) event->x;
		y = (gint) event->y;
		state = (GdkModifierType) event->state;
	} 

	cursor_x = x;
	cursor_y = y;

	/* update x/y display */
	nx = x2wvl( x );
	ny = y2flux( y );

	sprintf( label13_text, "%7.2f", nx );
	sprintf( label15_text, "%10.5e", ny );

	gtk_label_set_text(GTK_LABEL(label13), label13_text );
	gtk_label_set_text(GTK_LABEL(label15), label15_text );

	/* prepare for zoom box draw */
	if( one_pressed ) {
		xzoom2 = x;
		yzoom2 = y;
		erase_plot( widget );
	}
	if( cursor_type == CROSSHAIR_CURSOR ) {
		erase_plot( main_window );
	}
	return FALSE;
}

gboolean
on_drawingarea1_button_press_event     (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gtk_widget_grab_focus( widget );

	switch( (gint) event->button ) {
		case 1:
			one_pressed = TRUE;
			xzoom1 = xzoom2 = (gint) event->x;
			yzoom1 = yzoom2 = (gint) event->y;
			break;
		case 3:
			refline.b += ( y2flux( (gint) event->y ) - ((refline.m*(x2wvl((gint)event->x))) + refline.b) );
			erase_plot( main_window );
			break;
		default:
			break;
	}
	return FALSE;
}

gboolean
on_drawingarea1_button_release_event   (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gint x,y;

	if( one_pressed ) {
		x = (gint) event->x;
		y = (gint) event->y;

		xzoom2 = x;
		yzoom2 = y;

		if( (xzoom2 != xzoom1) && (yzoom2 != yzoom1 ) ){
			if( xzoom2 < xzoom1 ) {
				swap_ints( &xzoom1, &xzoom2 );
			}
			if( yzoom2 < yzoom1 ) {
				swap_ints( &yzoom1, &yzoom2 );
			}
			parea.wvlmax = x2wvl( xzoom2 );
			parea.wvlmin = x2wvl( xzoom1 );

			parea.fluxmin = y2flux( yzoom2 );
			parea.fluxmax = y2flux( yzoom1 );
			increment = (parea.fluxmax - parea.fluxmin)/100;
			erase_plot( main_window );
		}
		one_pressed = FALSE;
	}
	return FALSE;
}

gdouble x2wvl( gint x ) {
	return( ( pix2wvl.m*x) + pix2wvl.b );
}

gdouble y2flux( gint y ) {
	return( (pix2flux.m*y) + pix2flux.b );
}

void set_defaults() {
	struct spectdata	*sdata;
	GList				*node;
	gdouble				wmin,
						wmax,
						fmin,
						fmax;

	sdata = active_spectrum->data;

	wmin = sdata->ufile->x1;
	wmax = sdata->ufile->xn;
	fmin = sdata->ufile->ymin;
	fmax = sdata->ufile->ymax;

	for( node = spectra; node != NULL; node = g_list_next( node ) ) {
		sdata = node->data;
		if ( sdata->ufile->x1 < wmin ) {
			wmin = sdata->ufile->x1;
		}
		if ( sdata->ufile->xn > wmax ) {
			wmax = sdata->ufile->xn;
		}
		if( sdata->ufile->ymin < fmin ) {
			fmin = sdata->ufile->ymin;
		}
		if( sdata->ufile->ymax > fmax ) {
			fmax = sdata->ufile->ymax;
		}
	}
	parea.wvlmin = wmin;
	parea.wvlmax = wmax;
	parea.fluxmin = fmin - ((fmax-fmin)/10.0);
	parea.fluxmax = fmax + ((fmax-fmin)/10.0);

	if( refline.b == 10000.0 ) {
		refline.m = 0.;
		refline.b = parea.fluxmin + ((parea.fluxmax - parea.fluxmin)/2.);
	}
	increment = (parea.fluxmax - parea.fluxmin)/100;
	gtk_spin_button_set_value( GTK_SPIN_BUTTON (spinbutton1), a );
	gtk_spin_button_set_value( GTK_SPIN_BUTTON (spinbutton2), b );
	printed_header = FALSE;
}

void clean_up_and_quit() {
	struct spectdata *sdata;
	GList *node;

	for( node = spectra; node != NULL; node = g_list_next(node) ) {
		sdata = node->data;
		uni_free( sdata->ufile );
		free( sdata->coeff_b );
		free( sdata->coeff_c );
		free( sdata->filename );
		free( sdata );
	}
	g_list_free( spectra );

	free( blue_color );
	free( white_color );
	free( black_color );
	free( red_color );
	free( yellow_color );
	free( green_color );
	free( grey_color );
	free( purple_color );
	free( orange_color );

	g_list_free( ncolorlist );
	g_list_free( icolorlist );

	writelog( "Session Ended." );
	closelog();
	gtk_exit(0);
}

void update_labels() {
	/* draw the labels in the info box */
	gtk_spin_button_set_value( GTK_SPIN_BUTTON (xminspin), parea.wvlmin );
	gtk_spin_button_set_value( GTK_SPIN_BUTTON (xmaxspin), parea.wvlmax );
	sprintf( stattxt, "%e", parea.fluxmin );
	gtk_entry_set_text( GTK_ENTRY (yminentry), stattxt );
	sprintf( stattxt, "%e", parea.fluxmax );
	gtk_entry_set_text( GTK_ENTRY (ymaxentry), stattxt );
}

void swap_ints( gint* n1, gint* n2 ) {
	gint *tmp;

	tmp = malloc( sizeof( gint ) );

	*tmp = *n2;
	*n2 = *n1;
	*n1 = *tmp;

	free( tmp );
}

struct spectdata*  init_sdata( struct spectdata *sdata ) {
	if ( sdata == NULL ) {
		sdata = malloc( sizeof( struct spectdata ) );
		sdata->ufile = malloc( sizeof( struct unifile ) );
		sdata->coeff_b = NULL;
		sdata->coeff_c = NULL;
	}
	return( sdata );
}

void
on_drawingarea1_realize                (GtkWidget       *widget,
                                        gpointer         user_data)
{
	if( openlog( "log.txt" ) ) {
		g_print( "Can't open log file. Logging disabled.\n" );
	}
	writelog( "Session started..." );
	if( main_window == NULL ) {
		main_window = lookup_widget (GTK_WIDGET (widget), "main_window");

		xminspin = lookup_widget(GTK_WIDGET(main_window), "xminspin");
		xmaxspin = lookup_widget(GTK_WIDGET(main_window), "xmaxspin");
		yminentry = lookup_widget(GTK_WIDGET(main_window), "yminentry");
		ymaxentry = lookup_widget(GTK_WIDGET(main_window), "ymaxentry");

		label13 = lookup_widget(GTK_WIDGET(main_window), "label13");
		label15 = lookup_widget(GTK_WIDGET(main_window), "label15");

		spinbutton1 = lookup_widget(GTK_WIDGET(main_window), "spinbutton1");
		spinbutton2 = lookup_widget(GTK_WIDGET(main_window), "spinbutton2");
	}
	blue_color = malloc( sizeof( GdkColor ) );
	white_color = malloc( sizeof( GdkColor ) );
	black_color = malloc( sizeof( GdkColor ) );
	red_color = malloc( sizeof( GdkColor ) );
	yellow_color = malloc( sizeof( GdkColor ) );
	green_color = malloc( sizeof( GdkColor ) );
	grey_color = malloc( sizeof( GdkColor ) );
	purple_color = malloc( sizeof( GdkColor ) );
	orange_color = malloc( sizeof( GdkColor ) );

	gdk_color_parse( "blue",  blue_color);
	gdk_colormap_alloc_color( gdk_colormap_get_system(), blue_color, FALSE, TRUE );

	gdk_color_parse( "white",  white_color);
	gdk_colormap_alloc_color( gdk_colormap_get_system(), white_color, FALSE, TRUE );

	gdk_color_parse( "black",  black_color);
	gdk_colormap_alloc_color( gdk_colormap_get_system(), black_color, FALSE, TRUE );

	gdk_color_parse( "red",  red_color);
	gdk_colormap_alloc_color( gdk_colormap_get_system(), red_color, FALSE, TRUE );

	gdk_color_parse( "yellow",  yellow_color);
	gdk_colormap_alloc_color( gdk_colormap_get_system(), yellow_color, FALSE, TRUE );

	gdk_color_parse( "green",  green_color);
	gdk_colormap_alloc_color( gdk_colormap_get_system(), green_color, FALSE, TRUE );

	gdk_color_parse( "grey",  grey_color);
	gdk_colormap_alloc_color( gdk_colormap_get_system(), grey_color, FALSE, TRUE );

	gdk_color_parse( "purple",  purple_color);
	gdk_colormap_alloc_color( gdk_colormap_get_system(), purple_color, FALSE, TRUE );

	gdk_color_parse( "orange",  orange_color);
	gdk_colormap_alloc_color( gdk_colormap_get_system(), orange_color, FALSE, TRUE );

	ncolorlist = g_list_append( ncolorlist, blue_color );
	ncolorlist = g_list_append( ncolorlist, red_color );
	ncolorlist = g_list_append( ncolorlist, green_color );
	ncolorlist = g_list_append( ncolorlist, purple_color );
	ncolorlist = g_list_append( ncolorlist, orange_color );
	ncolorlist = g_list_append( ncolorlist, grey_color );

	icolorlist = g_list_append( icolorlist, yellow_color );
	icolorlist = g_list_append( icolorlist, red_color );
	icolorlist = g_list_append( icolorlist, green_color );
	icolorlist = g_list_append( icolorlist, purple_color );
	icolorlist = g_list_append( icolorlist, orange_color );
	icolorlist = g_list_append( icolorlist, grey_color );

	normal_colors();

	gdk_window_set_cursor ( widget->window, gdk_cursor_new( GDK_CROSSHAIR ) );

	refline.m = 0;
	refline.b = 10000.0;

	if( spectra == NULL ) {
		greyout( FALSE );
	}
	else  {
		greyout( TRUE );
	}
}


gboolean
on_drawingarea1_key_press_event        (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	gint			x,y;
	GdkModifierType	state;

	switch( event->keyval ) {
		case 65289: /* tab */
			if( active_spectrum->next != NULL )
				active_spectrum = active_spectrum->next;
			else
				active_spectrum = spectra;
			erase_plot( main_window );
			break;
		case 65361: /* left arrow */
			move_pointer( -1, 0 );
			break;
		case 65362: /* up arrow */
			move_pointer( 0, -1 );
			break;
		case 65363: /* right arrow */
			move_pointer( 1, 0 );
			break;
		case 65364: /* down arrow */
			move_pointer( 0, 1 );
			break;
		case 65360: /* home */
			increment_cont_inter( widget, 1 );
			break;
		case 65367: /* end */
			increment_cont_inter( widget, -1 );
			break;
		case 65535: /* delete */
			increment_cont_slope( widget, 1 );
			break;
		case 65365: /* page up */
			increment_cont_slope( widget, 1 );
			break;
		case 65366: /* page down */
			increment_cont_slope( widget, -1 );
			break;
		case 65379: /* insert */
			increment_cont_slope( widget, -1 );
			break;
		case 97: /* a */
			gdk_window_get_pointer( event->window, &x, &y, &state);
			if( x2wvl( x ) < b ) {
				a = x2wvl( x );
				gtk_spin_button_set_value( GTK_SPIN_BUTTON (spinbutton1), a );
			}
			break;
		case 98: /* b */
			gdk_window_get_pointer(event->window, &x, &y, &state);
			if( x2wvl( x ) > a ) {
				b = x2wvl( x );
				gtk_spin_button_set_value( GTK_SPIN_BUTTON (spinbutton2), b );
			}
			break;
		case 32: /* space */
			refline.m = 0;
			refline.b = parea.fluxmin + ((parea.fluxmax-parea.fluxmin)/2.);
			erase_plot( main_window );
			break;
		case 109: /* m */
			gdk_window_get_pointer( event->window, &x, &y, &state);
			mark = x2wvl( x );
			erase_plot( main_window );
			break;
		case 99: /* c */
			gdk_window_get_pointer( event->window, &x, &y, &state);
			struct spectdata *sdata = active_spectrum->data;
			gdouble w = x2wvl( x );
			if( w >= sdata->ufile->x1 && w <= sdata->ufile->xn ) {
				mark = centroid( sdata->ufile->x1, sdata->ufile->dx, sdata->ufile->data, sdata->ufile->num_elems, x2wvl( x ) );
				erase_plot( main_window );
			}
			else
				display_message( "Error: Centroid guess not in range of active spectrum" );
			break;
		default:
			sprintf( stattxt, "Sorry, that key doesn't do anything yet: %i.", event->keyval );
			writelog( stattxt );
			printed_header = FALSE;
			break;
	}

	return TRUE;
}

void increment_cont_slope( GtkWidget *widget, gint inc ) {
	double	ref_y1,
			ref_y2,
			scale_factor;

	ref_y1 = refline.m*parea.wvlmin + refline.b;
	ref_y2 = refline.m*parea.wvlmax + refline.b;

	scale_factor = (9*refline.m)+1;

	ref_y1 -= ( (increment*inc*scale_factor) );
	ref_y2 += ( (increment*inc*scale_factor) );
	refline.m = (ref_y2-ref_y1)/(parea.wvlmax-parea.wvlmin);
	refline.b = ref_y1 -(refline.m*parea.wvlmin );

	erase_plot( main_window );
}

void increment_cont_inter( GtkWidget *widget, gint inc ) {
	double	ref_y1,
			ref_y2;

	ref_y1 = refline.m*parea.wvlmin + refline.b;
	ref_y2 = refline.m*parea.wvlmax + refline.b;
	ref_y1 += (increment*inc);
	ref_y2 += (increment*inc);
	refline.m = (ref_y2-ref_y1)/(parea.wvlmax-parea.wvlmin);
	refline.b = ref_y1 -(refline.m*parea.wvlmin );
	erase_plot( main_window );
}

void
on_continuum_line1_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget * tb = lookup_widget (GTK_WIDGET (menuitem), "togglebutton1");
	if( gtk_check_menu_item_get_active(  GTK_CHECK_MENU_ITEM (menuitem ) ) == TRUE ) {
		show_guides = TRUE;
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON (tb), TRUE );
		update_status( "Guide lines revealed. ");
	}
	else {
		show_guides = FALSE;
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON (tb), FALSE );
		update_status( "Guide lines hidden. ");
	}
}


void
on_togglebutton1_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget * mi = lookup_widget (GTK_WIDGET (togglebutton), "continuum_line1");

	if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (togglebutton) )  == TRUE ) {
		show_guides = TRUE;
		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM (mi), TRUE );
		update_status( "Guide lines revealed. ");
	}
	else {
		show_guides = FALSE;
		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM (mi), FALSE );
		update_status( "Guide lines hidden. ");
	}
	erase_plot( main_window );
}


void
on_spinbutton1_value_changed           (GtkSpinButton   *spinbutton,
                                        gpointer         user_data)
{
	if( gtk_spin_button_get_value( spinbutton ) < b ) {
		a = gtk_spin_button_get_value( spinbutton );
	}
	else
		gtk_spin_button_set_value( GTK_SPIN_BUTTON (spinbutton), a );
	erase_plot( main_window );

}


void
on_spinbutton2_value_changed           (GtkSpinButton   *spinbutton,
                                        gpointer         user_data)
{
	if( gtk_spin_button_get_value( spinbutton ) > a ) {
		b = gtk_spin_button_get_value( spinbutton );
	}
	else
		gtk_spin_button_set_value( GTK_SPIN_BUTTON (spinbutton), b );
	erase_plot( main_window );

}


void
on_integrate1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	integrate( active_spectrum->data );

}


void
on_integrate_button_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	integrate( active_spectrum->data );

}

void integrate( struct spectdata *sdata ) {
	gdouble dx    		= 0.,	/* the interval between x values */
			carea 		= 0., 	/* area of region under the reference line */
			sarea 		= 0.,	/* area of the total region under the spectrum */
			ew    		= 0.,	/* equivalent width of spectrum feature */
			n     		= 0.,	/* counter that goes from a to b for integration */
			xcenter		= 0.,	/* x center of mass */
			ref_lvl     = 0.;	/* the lvl of the reference line at a+(b-a)/2 */
	gint	s			= 0;	/* sample number of current n */
	struct line wvl2sam;		/* converts wavelengths to samples */

	if( a==0 || b==0 ) {
		display_message( "Error: You must set the limits of integration first." );
		return;
	}

	dx = sdata->ufile->dx;

	/* Compute transform from wavlengths to samples */
	wvl2sam.m = (gdouble) ( (gdouble)(sdata->ufile->num_elems-1)/(sdata->ufile->xn-sdata->ufile->x1) );
	wvl2sam.b =  (gdouble) (sdata->ufile->num_elems - 1 ) - (wvl2sam.m*sdata->ufile->xn);

	/* do poor man's integral */
	for( n=a+dx; n<=b; n+=dx ) {
		/* get sample number for current wavelength */
		s = (gint)round((wvl2sam.m*n)+wvl2sam.b);
		if( s < 0 || s > sdata->ufile->num_elems ) {
			display_message( "Error: A or B outside range of active spectrum." );
			return;
		}

		/* compute area of entire region under spectrum. just width*height */
		sarea += ( dx * sdata->ufile->data[s] );

		/* compute area under reference line. refline.m and refline.b are
		 * slope and intercept of reference line in wavelengths vs. flux. 
		 * This takes width*height for a rectangle the same width as for sarea above.
		 * The height is computed with the slope and intercept computed when
		 * they set the line. We count on dx being sufficiently small.  */
		carea += ( dx * ((refline.m*(n)) + refline.b));
		xcenter += ( dx * ( (sdata->ufile->data[s] * n ) - ( dx * ((refline.m*(n)) + refline.b))));
	}

	/* Report on ref_lvl at center of integration limits */
	ref_lvl = ((refline.m*(((b-a)/2)+a)) + refline.b);

	/* compute the equivalent width using formula from Kris */
	ew = (sarea - carea)/(carea/(b-a));
	gdouble dy = sarea/(b-a);

	/* compute the center of mass for x */
	xcenter = xcenter/sarea;

	/* print header if we haven't already done so */
	if( ! printed_header ) {
		sprintf( stattxt, "%15s %7s %7s %8s %10s %7s %10s", "File", "A", "B", "E.W.", "Ref Lvl", "<X>", "<Y>" );
		writelog( stattxt );
		sprintf( stattxt, "%15s %7s %7s %8s %10s %7s %10s", "---------------", "-------", "-------", "--------", "----------", "-------", "----------" );
		writelog( stattxt );
		printed_header = TRUE;
	}

	/* print our integration results */
	sprintf( stattxt, "%15s %#7.2f %#7.2f %#8.4f %# 8.3e %#7.2f %#8.4e", sdata->filename, a, b, ew, ref_lvl, xcenter, dy );
	writelog( stattxt );

	GtkWidget *label30 = lookup_widget(GTK_WIDGET(main_window), "label30");
	GtkWidget *label31 = lookup_widget(GTK_WIDGET(main_window), "label31");
	sprintf( stattxt, "%# 7.2f", ew );
	gtk_label_set_text(GTK_LABEL(label30), stattxt );
	sprintf( stattxt, "%# 10.5e", ref_lvl );
	gtk_label_set_text(GTK_LABEL(label31), stattxt );

	/* update status bar on bottom of window */
	update_status( "Integration Complete" );
}

void
on_export1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	do_save_as();
}

void do_save_as() {
	gchar	label[64],
			 fname[PATH_MAX];
	gchar	*tmp_ptr;

	memset( fname, '\0', PATH_MAX );
	if ( dialog2 == NULL) {
		dialog2 = create_dialog2();
		gtk_window_set_transient_for(GTK_WINDOW (dialog2), GTK_WINDOW (main_window));
	}
	GtkWidget	*entry2 = lookup_widget(GTK_WIDGET( dialog2 ), "entry2");
	GtkWidget	*entry3 = lookup_widget(GTK_WIDGET( dialog2 ), "entry3");
	GtkWidget	*draw = lookup_widget(GTK_WIDGET( main_window ), "drawingarea1");
	const char	*stype = gtk_entry_get_text( GTK_ENTRY(lookup_widget(GTK_WIDGET( dialog2 ), "combo_entry1")) );

	tmp_ptr = strrchr( filename, '/' ); tmp_ptr++;
	strncpy( fname, tmp_ptr, PATH_MAX );
	strncat( fname, ".jpg", PATH_MAX-strlen(fname) );

	tmp_ptr = strrchr( fname, '.' ); tmp_ptr++;
	strncpy( tmp_ptr, stype, PATH_MAX );
	gtk_entry_set_text( GTK_ENTRY(lookup_widget(GTK_WIDGET( dialog2 ), "entry1")), fname );

	sprintf( label, "%i", draw->allocation.width );
	gtk_entry_set_text( GTK_ENTRY(entry2), label );

	sprintf( label, "%i", draw->allocation.height );
	gtk_entry_set_text( GTK_ENTRY(entry3), label );

	gtk_object_set_data(GTK_OBJECT (dialog2), MainWindowKey, main_window);
	gtk_window_present(GTK_WINDOW (dialog2));
}

void do_print() {
	gchar	label[64];

	if ( printdialog == NULL) {
		printdialog = create_printdialog();
		gtk_window_set_transient_for(GTK_WINDOW (printdialog), GTK_WINDOW (main_window));
	}
	GtkWidget	*entry6 = lookup_widget(GTK_WIDGET( printdialog ), "entry6");
	GtkWidget	*entry7 = lookup_widget(GTK_WIDGET( printdialog ), "entry7");
	GtkWidget	*draw = lookup_widget(GTK_WIDGET( main_window ), "drawingarea1");

	sprintf( label, "%i", draw->allocation.width );
	gtk_entry_set_text( GTK_ENTRY(entry6), label );

	sprintf( label, "%i", draw->allocation.height );
	gtk_entry_set_text( GTK_ENTRY(entry7), label );

	gtk_object_set_data(GTK_OBJECT (printdialog), MainWindowKey, main_window);
	gtk_window_present(GTK_WINDOW (printdialog));
}


void
on_cancelbutton1_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide (gtk_widget_get_toplevel (GTK_WIDGET (button)));

}


void
on_okbutton1_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	const gchar	*stype, *sname, *w, *h;
	gint		width, height;
	GtkWidget	*type = lookup_widget(GTK_WIDGET( dialog2 ), "combo_entry1");
	GtkWidget	*name = lookup_widget(GTK_WIDGET( dialog2 ), "entry1");
	GtkWidget	*entry2 = lookup_widget(GTK_WIDGET( dialog2 ), "entry2");
	GtkWidget	*entry3 = lookup_widget(GTK_WIDGET( dialog2 ), "entry3");
                                                                                     
	dialog2 = gtk_widget_get_toplevel (GTK_WIDGET (button));
	gtk_widget_hide( dialog2 );

	stype = gtk_entry_get_text( GTK_ENTRY(type) );
	sname = gtk_entry_get_text( GTK_ENTRY(name) );
	w     = gtk_entry_get_text( GTK_ENTRY(entry2) );
	h     = gtk_entry_get_text( GTK_ENTRY(entry3) );

	width  = atoi( w );
	height = atoi( h );

	if( width < 1 || height < 1 ) {
		display_message( "width and height must be > 0" );
		return;
	}
	save_image( sname, stype, width, height );

	return;
}

int normal_colors() {
	zoom_color	= red_color;
	axes_color	= black_color;
	mark_color	= orange_color;
	cursor_color	= grey_color;
	ab_color	= red_color;
	ref_color	= black_color;
	bg_color	= white_color;

	colorlist = ncolorlist;

	return( 1 );
}

int inverted_colors() {
	zoom_color		= red_color;
	axes_color		= white_color;
	mark_color		= green_color;
	cursor_color	= orange_color;
	ab_color		= red_color;
	ref_color		= white_color;
	bg_color		= black_color;

	colorlist = icolorlist;

	return( 1 );
}

void
on_invert1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget * tb = lookup_widget (GTK_WIDGET (menuitem), "togglebutton3");
	if( gtk_check_menu_item_get_active(  GTK_CHECK_MENU_ITEM (menuitem ) ) == TRUE ) {
		inverted_colors();
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON (tb), TRUE );
		update_status( "Colors inverted" );
	}
	else {
		normal_colors();
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON (tb), FALSE );
		update_status( "Colors not inverted" );
	}
	erase_plot( main_window );
}


void
on_overview1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if ( help_window == NULL) {
		help_window = create_help_window();
		gtk_window_set_transient_for(GTK_WINDOW (help_window), GTK_WINDOW (main_window));
	}

	gtk_object_set_data(GTK_OBJECT (help_window), MainWindowKey, main_window);
	gtk_window_present(GTK_WINDOW (help_window));
}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if ( dialog3 == NULL) {
		dialog3 = create_dialog3();
		gtk_window_set_transient_for(GTK_WINDOW (dialog3), GTK_WINDOW (main_window));
	}

	gtk_object_set_data(GTK_OBJECT (dialog3), MainWindowKey, main_window);
	gtk_window_present(GTK_WINDOW (dialog3));
}


void
on_togglebutton3_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget * mi = lookup_widget (GTK_WIDGET (togglebutton), "invert1");

	if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (togglebutton) )  == TRUE ) {
		inverted_colors();
		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM (mi), TRUE );
		update_status( "Colors inverted" );
	}
	else {
		normal_colors();
		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM (mi), FALSE );
		update_status( "Colors not inverted" );
	}
	erase_plot( main_window );
}


void
on_button4_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide( dialog3 );

}


gboolean
on_window1_delete_event                (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{

	gtk_widget_hide( help_window );
	return TRUE;
}

void
on_cursor_mode1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget * tb = lookup_widget (GTK_WIDGET (menuitem), "cursortoggle");
	if( gtk_check_menu_item_get_active(  GTK_CHECK_MENU_ITEM (menuitem ) ) == TRUE ) {
		cursor_type = CROSSHAIR_CURSOR;
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON (tb), TRUE );
		update_status( "Using normal cursor" );
	}
	else {
		cursor_type = NORMAL_CURSOR;
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON (tb), FALSE );
		update_status( "Using crosshair cursor" );
	}

}


void
on_cursortoggle_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget * mi = lookup_widget (GTK_WIDGET (togglebutton), "cursor_mode1");

	if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (togglebutton) )  == TRUE ) {
		cursor_type = CROSSHAIR_CURSOR;
		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM (mi), TRUE );
		update_status( "Using normal cursor" );
	}
	else {
		cursor_type = NORMAL_CURSOR;
		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM (mi), FALSE );
		update_status( "Using crosshair cursor" );
	}
	erase_plot( main_window );
}

void save_image( char* sname, char* stype, int width, int height ) {
	GdkPixbuf	*pixbuf;
	GError		*error = NULL;

	GtkWidget	*draw = lookup_widget(GTK_WIDGET( main_window ), "drawingarea1");
	GdkPixmap *pixmap = gdk_pixmap_new( draw->window, width, height, -1);
	draw_plot( GDK_DRAWABLE(pixmap) );

	pixbuf = gdk_pixbuf_get_from_drawable( NULL, pixmap, gdk_colormap_get_system(), 0, 0, 0, 0, -1, -1);
	if ( pixbuf != NULL ) {
		if( strncmp( stype, "jpeg", 16 ) == 0 )
			gdk_pixbuf_save( GDK_PIXBUF(pixbuf), sname, stype, &error, "quality", "100", NULL);
		else if( strncmp( stype, "png", 16 ) == 0 )
			gdk_pixbuf_save( GDK_PIXBUF(pixbuf), sname, stype, &error, NULL);
		else if( strncmp( stype, "eps", 16 ) == 0 ) {
			gdk_pixbuf_save( GDK_PIXBUF(pixbuf), ".tmp.jpg", "jpeg", &error, "quality", "100", NULL);
			img2ps( sname, ".tmp.jpg" );
			unlink( ".tmp.jpg" );
		}
	}
	else
		display_message( "pixbuf was NULL -- no file saved." );

	return;
}

void
on_print1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	do_print();

}

void
on_printokbutton_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	pid_t	pid;
	int rtn;
	const gchar	*w, *h;
	static char * env[] = { 
    	NULL
	};
  	char *args[] = { 
		"/usr/bin/lp",
    	PRINT_TMP_FILE,
		NULL
	};

	gint		width, height;
	GtkWidget	*entry6 = lookup_widget(GTK_WIDGET( printdialog ), "entry6");
	GtkWidget	*entry7 = lookup_widget(GTK_WIDGET( printdialog ), "entry7");
                                                                                     
	printdialog = gtk_widget_get_toplevel (GTK_WIDGET (button));
	gtk_widget_hide( printdialog );

	w     = gtk_entry_get_text( GTK_ENTRY(entry6) );
	h     = gtk_entry_get_text( GTK_ENTRY(entry7) );

	width  = atoi( w );
	height = atoi( h );

	if( width < 1 || height < 1 ) {
		display_message( "width and height must be > 0" );
		return;
	}
	save_image( PRINT_TMP_FILE, "eps", width, height );

	pid = fork();
	if( pid == 0 )  { /* we're the child */
		rtn = execve( "/usr/bin/lp",  args, env );
		printf( "Printing failed. Return code from execve is %i\n", rtn );
	}
	if( pid > 0 ) { /* we're the parent */
		sleep( 3 );
		unlink( PRINT_TMP_FILE );
	}

	return;
}

void move_pointer( int xinc, int yinc ) {
	GtkWidget *draw = lookup_widget(GTK_WIDGET(main_window), "drawingarea1");
	XWarpPointer( GDK_WINDOW_XDISPLAY(draw->window), None, None, 0, 0, 0, 0, xinc, yinc );
}

void draw_spectrum( struct spectdata *sdata, GdkDrawable *widget, GdkGC *gc, int xrange, int yrange, struct line xform ) {
	gdouble		w=0.0;
	gint		x, y, x2, y2, n, s1, sn;
	struct line	wvl2sam,
				sam2wvl;

	/* Compute transform from wavlengths to samples */
	wvl2sam.m = (gdouble) ( ((gdouble)sdata->ufile->num_elems-1)/(sdata->ufile->xn-sdata->ufile->x1) );
	wvl2sam.b =  (gdouble) (sdata->ufile->num_elems-1) - (wvl2sam.m*sdata->ufile->xn);

	/* Compute transform from samples to wavlengths */
	sam2wvl.m = (sdata->ufile->xn-sdata->ufile->x1)/(gdouble)(sdata->ufile->num_elems-1);
	sam2wvl.b = sdata->ufile->xn - (sam2wvl.m*(gdouble) (sdata->ufile->num_elems-1));

	s1 = (gint)round((wvl2sam.m*parea.wvlmin)+wvl2sam.b);
	if( s1 < 0 )
		s1 = 0;

	sn = (gint)round((wvl2sam.m*parea.wvlmax)+wvl2sam.b);
	if( sn > (sdata->ufile->num_elems-1) )
		sn = sdata->ufile->num_elems-1;

	if( sn-s1 <= 0 ) return;

	if( xrange/(sn-s1) <=1 ) {
		for( n=s1; n<sn; ++n ) {
			y  = (flux2pix.m*( sdata->ufile->data[n] ))+flux2pix.b;
			if( y < BORDER )
				y=BORDER;
			if( y >yrange+BORDER )
				y = yrange+BORDER;
			y2 = (flux2pix.m*( sdata->ufile->data[n+1] ))+flux2pix.b;
			if( y2 < BORDER )
				y2=BORDER;
			if( y2 >yrange+BORDER )
				y2 = yrange+BORDER;

			x  = (wvl2pix.m*( (sam2wvl.m*n)+sam2wvl.b ))+wvl2pix.b;
			if( x < BORDER )
				x=BORDER;
			if( x > xrange+BORDER )
				x = xrange+BORDER;
	
			x2 = (wvl2pix.m*( (sam2wvl.m*(n+1))+sam2wvl.b ))+wvl2pix.b;
			if( x2 < BORDER )
				x2=BORDER;
			if( x2 > xrange+BORDER )
				x2 = xrange+BORDER;
	
			if( x > BORDER && x2 < xrange+BORDER ) {
				if( ! ( (y == y2) && ( y2==BORDER || y2==BORDER+yrange ) ) ) {
					if( ! ( (x == x2) && ( x2==BORDER || x2==BORDER+yrange ) ) ) {
		  				gdk_draw_line( GDK_DRAWABLE( widget ), gc, x, y, x2, y2 );
						if( ( sdata == active_spectrum->data ) && ( g_list_length( spectra ) > 1 ) ) {
  							gdk_draw_line( GDK_DRAWABLE( widget ), gc, x, y-1, x2, y2-1 );
  							gdk_draw_line( GDK_DRAWABLE( widget ), gc, x, y+1, x2, y2+1 );
						}
					}
				}
			}
		}
	}
	else {
		for( x=BORDER; x<(xrange+BORDER); ++x ) {
			w = x2wvl( x );
			if( w >= sdata->ufile->x1 && w <= sdata->ufile->xn ) {
				y  = (gint)round( ( x2y( w, sdata->ufile->x1, sdata->ufile->dx, sdata ) * xform.m) + xform.b);
	
				if( y < BORDER )
					y=BORDER;
				if( y >yrange+BORDER )
					y = yrange+BORDER;
				x2=x+1;
				w = x2wvl( x2 );
				if( w >= sdata->ufile->x1 && w <= sdata->ufile->xn ) {
					y2  = (gint)round( ( x2y( w, sdata->ufile->x1, sdata->ufile->dx, sdata ) * xform.m) + xform.b);
					if( y2 < BORDER )
						y2=BORDER;
					if( y2 >yrange+BORDER )
						y2 = yrange+BORDER;
					if( ! ( (y == y2) && ( y2==BORDER || y2==BORDER+yrange ) ) ) {
  						gdk_draw_line( GDK_DRAWABLE( widget ), gc, x, y, x2, y2 );
						if( sdata == active_spectrum->data ) {
  							gdk_draw_line( GDK_DRAWABLE( widget ), gc, x, y-1, x2, y2-1 );
  							gdk_draw_line( GDK_DRAWABLE( widget ), gc, x, y+1, x2, y2+1 );
						}
					}
				}
			}
		}
/*
		for( n=s1; n<sn; ++n ) {
			y  = (flux2pix.m*( sdata->ufile->data[n] ))+flux2pix.b;
			if( y < BORDER )
				y=BORDER;
			if( y >yrange+BORDER )
				y = yrange+BORDER;
			y2 = (flux2pix.m*( sdata->ufile->data[n+1] ))+flux2pix.b;
			if( y2 < BORDER )
				y2=BORDER;
			if( y2 >yrange+BORDER )
				y2 = yrange+BORDER;

			x  = (wvl2pix.m*( (sam2wvl.m*(n))+sam2wvl.b ))+wvl2pix.b;
			gdk_draw_rectangle( GDK_DRAWABLE( widget ), gc, TRUE, x-3, y-3, 6, 6 );
		}
 */
	}
}

gboolean
on_close_dialog_delete_event           (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	gtk_widget_hide( close_dialog );
 	return TRUE;
}


void
on_s1_close_button_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{

	free_node( g_list_nth( spectra, 0 ) );
	clear_button( 1, button );
}


void
on_s2_close_button_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	free_node( g_list_nth( spectra, 1 ) );
	clear_button( 2, button );

}


void
on_s3_close_button_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	free_node( g_list_nth( spectra, 2 ) );
	clear_button( 3, button );

}


void
on_s4_close_button_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	free_node( g_list_nth( spectra, 3 ) );
	clear_button( 4, button );

}


void
on_s5_close_button_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	free_node( g_list_nth( spectra, 4 ) );
	clear_button( 5, button );

}


void
on_s6_close_button_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	free_node( g_list_nth( spectra, 5 ) );
	clear_button( 6, button );

}


void
on_close1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gint 				x;
	gchar				labeltxt[PATH_MAX],
						item[16],
						butt[16];
	struct spectdata	*sdata = NULL;

	if ( close_dialog == NULL) {
		close_dialog = create_close_dialog();
		gtk_window_set_transient_for(GTK_WINDOW (close_dialog), GTK_WINDOW (main_window));
	}

	for( x=0; x<6; ++x ) {
		sprintf( item, "s%i_close_label", x+1 );
		sprintf( butt, "s%i_close_button", x+1 );

		GtkWidget *label = lookup_widget( GTK_WIDGET( close_dialog ), item );
		GtkWidget *button = lookup_widget( GTK_WIDGET( close_dialog ), butt );

		sdata = g_list_nth_data( spectra, x );
		if( sdata != NULL ) {
			sprintf( labeltxt, "%s", sdata->filename );
			gtk_label_set_text(GTK_LABEL(label), labeltxt );
			gtk_widget_set_sensitive( label, TRUE );
			gtk_widget_set_sensitive( button, TRUE );
		}
		else {
			sprintf( labeltxt, "%s", "Available" );
			gtk_label_set_text(GTK_LABEL(label), labeltxt );
			gtk_widget_set_sensitive( label, FALSE );
			gtk_widget_set_sensitive( button, FALSE );
		}
	}
	gtk_object_set_data(GTK_OBJECT (close_dialog), MainWindowKey, main_window);
	gtk_window_present(GTK_WINDOW (close_dialog));
}

void free_node( GList *node ) {
	struct spectdata	*sdata;

	if( node == active_spectrum ) {
		if( node->next != NULL )
			active_spectrum = node->next;
		else if( node->prev != NULL )
			active_spectrum = node->prev;
		else
			active_spectrum = NULL;
	}
	sdata = node->data;
	spectra = g_list_remove_link( spectra, node );
	uni_free( sdata->ufile );
	free( sdata->coeff_b );
	free( sdata->coeff_c );
	free( sdata->filename );
	free( sdata );
	g_list_free( node );
}

void clear_button( gint n, GtkButton *button ) {
	gchar				labeltxt[16],
						item[16],
						butt[16];

	sprintf( item, "s%i_close_label", n );
	sprintf( butt, "s%i_close_button", n );

	GtkWidget *l = lookup_widget( GTK_WIDGET( close_dialog ), item );
	GtkWidget *b = lookup_widget( GTK_WIDGET( close_dialog ), butt );

	sprintf( labeltxt, "%s", "Available" );
	gtk_label_set_text(GTK_LABEL(l), labeltxt );
	gtk_widget_set_sensitive( l, FALSE );
	gtk_widget_set_sensitive( b, FALSE );

	gtk_widget_hide (gtk_widget_get_toplevel (GTK_WIDGET (button)));
	if( g_list_length( spectra ) > 0 )
		on_close1_activate(NULL, NULL);
}

void update_status( char *message ) {
	gtk_statusbar_pop (GTK_STATUSBAR (statusbar), 1);
	gtk_statusbar_push (GTK_STATUSBAR (statusbar), 1,  message );
}

void do_fwhm() {
	struct full_width_half_max fwhm;
	struct spectdata *sdata;

	if( mark <= 0 )
		display_message( "Set mark (keyboard 'm') near max or min of feature first and set reference line to intersect feature." );
	else {
		sdata = active_spectrum->data;
		fwhm = calc_fwhm( refline, mark, sdata );
		if( fwhm.value == -1.0 )
			display_message( "Error calculating FWHM. Check reference line and mark line and try again." );
		else {
			sprintf( stattxt, "File %s: FWHM=%9.4f. Max wvl= %9.4f. Center wvl= %9.4f", sdata->filename, fwhm.value, fwhm.max_wvl, fwhm.center );
			writelog( stattxt );
			draw_fwhm_line( fwhm.left, fwhm.right, active_spectrum->data );
			mark_point( fwhm.max_wvl, fwhm.half_max+( (refline.m*fwhm.max_wvl)+refline.b ) );
			mark_point( fwhm.left, x2y( fwhm.left, sdata->ufile->x1, sdata->ufile->dx, sdata ) );
			mark_point( fwhm.right, x2y( fwhm.right, sdata->ufile->x1, sdata->ufile->dx, sdata ) );
			report_fwhm( fwhm );
		}
	}
	return;
}

void draw_fwhm_line( gdouble left, gdouble right, struct spectdata *sdata ) {
	gint x, y, x2, y2;
	gdouble f1, f2;

	x  = (gint) round( wvl2pix.m*left + wvl2pix.b );
	x2 = (gint) round( wvl2pix.m*right + wvl2pix.b );
	f1 = x2y( left, sdata->ufile->x1, sdata->ufile->dx, sdata );
	f2 = x2y( right, sdata->ufile->x1, sdata->ufile->dx, sdata );
	y  = (gint) round( flux2pix.m*f1 + flux2pix.b );
	y2 = (gint) round( flux2pix.m*f2 + flux2pix.b );

	GtkWidget *draw = lookup_widget(GTK_WIDGET(main_window), "drawingarea1");
	gdk_gc_set_foreground( draw->style->fg_gc[GTK_WIDGET_STATE (draw)], zoom_color);
	gdk_gc_set_line_attributes( draw->style->fg_gc[GTK_WIDGET_STATE (draw)], 1, GDK_LINE_ON_OFF_DASH, GDK_CAP_NOT_LAST, GDK_JOIN_MITER);
  	gdk_draw_line( GDK_DRAWABLE( draw->window ), draw->style->fg_gc[GTK_WIDGET_STATE (draw)], x, y, x2, y2 );
	gdk_gc_set_line_attributes( draw->style->fg_gc[GTK_WIDGET_STATE (draw)], 1, GDK_LINE_SOLID, GDK_CAP_NOT_LAST, GDK_JOIN_MITER);
	gdk_gc_set_foreground( draw->style->fg_gc[GTK_WIDGET_STATE (draw)], black_color);
	
}

void mark_point( gdouble wvl, gdouble flux ) {
	gint x, y;

	x = (gint) round( wvl2pix.m*wvl + wvl2pix.b );
	y = (gint) round( flux2pix.m*flux + flux2pix.b );

	GtkWidget *draw = lookup_widget(GTK_WIDGET(main_window), "drawingarea1");
	gdk_gc_set_foreground( draw->style->fg_gc[GTK_WIDGET_STATE (draw)], zoom_color);
	gdk_draw_rectangle( GDK_DRAWABLE( draw->window ), draw->style->fg_gc[GTK_WIDGET_STATE (draw)], TRUE, x-3, y-3, 6, 6 );
	gdk_gc_set_foreground( draw->style->fg_gc[GTK_WIDGET_STATE (draw)], black_color);
	
}

void
on_fwhm1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	do_fwhm();
}


void
on_fwhm_button_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	do_fwhm();
}

void report_fwhm( struct full_width_half_max fwhm ) {
	gchar	labeltxt[64];

	GtkWidget *f = lookup_widget( main_window, "fwhm_value" );
	GtkWidget *c = lookup_widget( main_window, "fwhm_center_value" );

	sprintf( labeltxt, "%9.4f", fwhm.value );
	gtk_label_set_text(GTK_LABEL(f), labeltxt );
	sprintf( labeltxt, "%9.4f", fwhm.center );
	gtk_label_set_text(GTK_LABEL(c), labeltxt );

	return;
}

void display_message( char* message ) {
	if ( message_dialog == NULL) {
		message_dialog = create_message_dialog();
		gtk_window_set_transient_for(GTK_WINDOW (message_dialog), GTK_WINDOW (main_window));
	}
	gtk_label_set_text(GTK_LABEL(lookup_widget( message_dialog, "message" )), message );
	update_status( message );
	writelog( message );
	printed_header = FALSE;

	gtk_object_set_data(GTK_OBJECT (message_dialog), MainWindowKey, main_window);
	gtk_window_present(GTK_WINDOW (message_dialog));
}


gboolean
on_message_dialog_delete_event         (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	gtk_widget_hide( message_dialog );
	return TRUE;
}


void
on_msg_dialog_ok_button_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide( message_dialog );

}

void
on_xminspin_value_changed              (GtkSpinButton   *spinbutton,
                                        gpointer         user_data)
{
	parea.wvlmin = gtk_spin_button_get_value( GTK_SPIN_BUTTON( spinbutton ) );
	erase_plot( main_window );

}

void
on_xmaxspin_value_changed              (GtkSpinButton   *spinbutton,
                                        gpointer         user_data)
{
	parea.wvlmax = gtk_spin_button_get_value( GTK_SPIN_BUTTON( spinbutton ) );
	erase_plot( main_window );

}

gboolean
on_yminentry_focus_out_event           (GtkWidget       *widget,
                                        GdkEventFocus   *event,
                                        gpointer         user_data)
{

	parea.fluxmin = strtod( gtk_entry_get_text( GTK_ENTRY( yminentry ) ), NULL );
	erase_plot( main_window );
	return FALSE;
}


gboolean
on_yminentry_key_press_event           (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{

	if( event->keyval == 65293 ) {
		parea.fluxmin = strtod( gtk_entry_get_text( GTK_ENTRY( yminentry ) ), NULL );
		erase_plot( main_window );
	}
	return FALSE;
}


gboolean
on_ymaxentry_focus_out_event           (GtkWidget       *widget,
                                        GdkEventFocus   *event,
                                        gpointer         user_data)
{

	parea.fluxmax = strtod( gtk_entry_get_text( GTK_ENTRY( ymaxentry ) ), NULL );
	erase_plot( main_window );
	return FALSE;
	return FALSE;
}


gboolean
on_ymaxentry_key_press_event           (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	if( event->keyval == 65293 || event->keyval == 65421 ) {
		parea.fluxmax = strtod( gtk_entry_get_text( GTK_ENTRY( ymaxentry ) ), NULL );
		erase_plot( main_window );
	}
	return FALSE;
}

void
on_combo_entry1_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{
	const char	*stype,
				*sname;
	char		*tmp_ptr,
				fname[PATH_MAX];

	memset( fname, '\0', PATH_MAX );

	stype = gtk_entry_get_text( GTK_ENTRY(lookup_widget(GTK_WIDGET( dialog2 ), "combo_entry1")) );
	sname = gtk_entry_get_text( GTK_ENTRY(lookup_widget(GTK_WIDGET( dialog2 ), "entry1")) );
	if( stype[0] != '\0' ) {
		strncpy( fname, sname, PATH_MAX );
		tmp_ptr = strrchr( fname, '.' ); tmp_ptr++;
		strncpy( tmp_ptr, stype, PATH_MAX );
		gtk_entry_set_text( GTK_ENTRY(lookup_widget(GTK_WIDGET( dialog2 ), "entry1")), fname );
	}

}


gboolean
on_scale_dialog_delete_event           (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{

	gtk_widget_hide( scale_dialog );
	return FALSE;
}


void
on_scale_cancelbutton_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide( scale_dialog );

}


void
on_scale_okbutton_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide( scale_dialog );
	// do stuff

}


void
on_scale_to1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gint 				x;
	gchar				labeltxt[PATH_MAX];
	struct spectdata	*sdata = NULL;
	GtkWidget			*combo;
	GList				*items = NULL;


	if ( scale_dialog == NULL) {
		scale_dialog = create_scale_dialog();
		gtk_window_set_transient_for(GTK_WINDOW (scale_dialog), GTK_WINDOW (main_window));
	}
	combo = lookup_widget( GTK_WIDGET( scale_dialog ), "scale_combo" );

	for( x=0; x<6; ++x ) {
		sdata = g_list_nth_data( spectra, x );
		if( sdata != NULL && sdata != active_spectrum->data ) {
			g_print( "filename is %s\n", sdata->filename );
			sprintf( labeltxt, "%s", sdata->filename );
  			items = g_list_append ( items, labeltxt );
		}
	}
	g_print( "length is %i\n", g_list_length( items ) );
	gtk_combo_set_popdown_strings (GTK_COMBO (combo), items);
	gtk_object_set_data(GTK_OBJECT (scale_dialog), MainWindowKey, main_window);
	gtk_window_present(GTK_WINDOW (scale_dialog));
}

