#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "centroid.h"

double centroid( double xmin, double dx, double *data, int num_samples, double guess ) {
	int		x;
	double	**ndata, y, ym, yp, min, step;

	step  = dx * 4.0;

	ndata = malloc( num_samples * sizeof( int ) );

	for( x=0; x < num_samples; ++x ) {
		ndata[x] = malloc( 2 * sizeof( double ) );
		ndata[x][0] = xmin + (dx*(double)x);
		ndata[x][1] = data[x];
	}

	y  = get_diff(  0, step, guess, ndata, num_samples );
	ym = get_diff( -1, step, guess, ndata, num_samples );
	yp = get_diff(  1, step, guess, ndata, num_samples );

	min = guess + ( 0.5 * (ym-yp)/(ym-(2*y)+yp) * step );

	for( x=0; x < num_samples; ++x ) {
		free( ndata[x] );
	}
	free( ndata );
	return( min );
}

/* sign should be -1, 0 or 1 only */
double get_diff( int sign, double step, double g, double **data, int num_samples ) {
	int x, prev, next, slope;
	double l, r, y=0;

	g += ((double)sign*step);

	for( x=1; x<=8; ++x ) {
		prev = get_bound( g - (step*x), data, num_samples ); 
		next = prev+1;
		slope = calc_slope( data[prev][0], data[prev][1], data[next][0], data[next][1] );
		l = calc_y( slope, data[prev][0], data[next][1] - ( slope*data[next][0] ) );

		prev = get_bound( g + (step*x), data, num_samples ); 
		next = prev+1;
		slope = calc_slope( data[prev][0], data[prev][1], data[next][0], data[next][1] );
		r = calc_y( slope, data[prev][0], data[next][1] - ( slope*data[next][0] ) );
		y += pow( r-l, 2.0 );
	}
	return( y );
}

double calc_slope( double x1, double y1, double x2, double y2 ) {
	double slope = (y2-y1)/(x2-x1);
	return( slope );
}

double calc_y( double m, double x, double b ) {
	double y;

	y = (m*x)+b;
	return( y );
}

double get_bound( double t, double **data, int num_points ) {
	int x;

	for( x=0; x<num_points; ++x ) {
		if( ( data[x][0] <=t ) && ( data[x+1][0] >= t ) ) {
			return( x );
		}
	}
	return( 0.0 );
}
