#include "cspec.h"
#include "fwhm.h"
#include "genbc.h"

struct full_width_half_max calc_fwhm( struct line refline, double mark, struct spectdata *sdata ) {
	struct full_width_half_max	fwhm;
	double						diff,
								max,
								current_wvl,
								last_diff=0.0,
								last_wvl=0.0,
								bee,
								aye,
								tan_theta,
								a_prime;
	int							is_positive = 1,
								sign_switched = 0;

	double dx = (sdata->ufile->dx)/16.0;

	current_wvl = mark;
	fwhm.max_wvl = current_wvl;
	max = x2y( current_wvl, sdata->ufile->x1, sdata->ufile->dx, sdata ) - ( (refline.m*current_wvl)+refline.b );
	if( max > 0 )
		is_positive = 1;
	else
		is_positive = 0;

	while( ! sign_switched ) {
		current_wvl = current_wvl - dx;
		if (current_wvl < sdata->ufile->x1 || current_wvl > sdata->ufile->xn ) {
			fwhm.value = -1.0;
			return( fwhm );
		}
		diff = x2y( current_wvl, sdata->ufile->x1, sdata->ufile->dx, sdata ) - ( (refline.m*current_wvl)+refline.b );
		if( ( is_positive && diff < 0 ) || ( ! is_positive && diff > 0 ) ) {
			sign_switched = 1;
		}
		else {
			if( fabs( diff ) > fabs( max ) ) {
				max = diff;
				fwhm.max_wvl = current_wvl;
			}
		}
	}

	sign_switched = 0;
	current_wvl = mark;

	while( ! sign_switched ) {
		current_wvl = current_wvl + dx;
		if (current_wvl < sdata->ufile->x1 || current_wvl > sdata->ufile->xn ) {
			fwhm.value = -1.0;
			return( fwhm );
		}
		diff = x2y( current_wvl, sdata->ufile->x1, sdata->ufile->dx, sdata ) - ( (refline.m*current_wvl)+refline.b );
		if( ( is_positive && diff < 0 ) || ( ! is_positive && diff > 0 ) ) {
			sign_switched = 1;
		}
		else {
			if( fabs( diff ) > fabs( max ) ) {
				max = diff;
				fwhm.max_wvl = current_wvl;
			}
		}
	}
	fwhm.half_max = max/2.0;

	sign_switched = 0;
	current_wvl = fwhm.max_wvl;

	max = x2y( current_wvl, sdata->ufile->x1, sdata->ufile->dx, sdata ) - fwhm.half_max - ( (refline.m*current_wvl)+refline.b );
	if( max > 0 )
		is_positive = 1;
	else
		is_positive = 0;

	while( ! sign_switched ) {
		diff = x2y( current_wvl, sdata->ufile->x1, sdata->ufile->dx, sdata ) - fwhm.half_max - ( (refline.m*current_wvl)+refline.b );
		if( ( is_positive && diff < 0 ) || ( is_positive == 0 && diff > 0 ) ) {
			sign_switched = 1;
			bee = fabs( last_diff ) + fabs( diff );
			aye =  last_wvl - current_wvl;
			tan_theta = bee/aye;
			a_prime = diff/tan_theta;
			if( is_positive ) a_prime = a_prime*(-1.0);
			fwhm.left = current_wvl + a_prime;
		}
		else {
			last_wvl = current_wvl;
			last_diff = diff;
		}
		current_wvl = current_wvl - dx;
	}

	sign_switched = 0;
	current_wvl = fwhm.max_wvl;

	while( ! sign_switched ) {
		diff = x2y( current_wvl, sdata->ufile->x1, sdata->ufile->dx, sdata ) - fwhm.half_max - ( (refline.m*current_wvl)+refline.b );
		if( ( is_positive && diff < 0 ) || ( is_positive == 0 && diff > 0 ) ) {
			sign_switched = 1;
			bee = fabs( last_diff ) + fabs( diff );
			aye = current_wvl - last_wvl;
			tan_theta = bee/aye;
			a_prime = diff/tan_theta;
			if( is_positive ) a_prime = a_prime*(-1.0);
			fwhm.right = current_wvl - a_prime;
		}
		else {
			last_wvl = current_wvl;
			last_diff = diff;
		}
		current_wvl = current_wvl + dx;
	}

	fwhm.value = fwhm.right - fwhm.left;
	fwhm.center = (fwhm.value/2.0)+fwhm.left;

	return( fwhm );
}
